<?php

namespace Makecommerce\Ecommerce\Model;

use Magento\Checkout\Api\GuestPaymentInformationManagementInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Store\Model\StoreManagerInterface;
use Makecommerce\Ecommerce\Api\Data\CustomerInterface;
use Makecommerce\Ecommerce\Api\Data\TransactionInterface;
use Makecommerce\Ecommerce\Api\GuestPaymentInformationAdapterInterface;
use Makecommerce\Ecommerce\Api\TransactionManagerInterface;

class GuestPaymentInformationAdapter implements GuestPaymentInformationAdapterInterface
{
    // @codingStandardsIgnoreStart
    /**
     * @var GuestPaymentInformationManagementInterface
     */
    protected $guestPaymentInformationManagement;

    /**
     * @var OrderRepository
     */
    protected $orderRepository;

    /**
     * @var TransactionManagerInterface
     */
    protected $transactionManager;

    /**
     * @var AdapterResponse
     */
    protected $adapterResponseFactory;

    /**
     * @var TransactionInterface
     */
    protected $transaction;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var CustomerInterface
     */
    protected $customer;

    /**
     * @var \Magento\Framework\Locale\ResolverInterface
     */
    protected $resolver;

    /**
     * @var CartRepositoryInterface
     */
    protected $quoteRepository;
    // @codingStandardsIgnoreEnd

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    private $orderCollectionFactory;

    /**
     * @var \Magento\Sales\Api\OrderManagementInterface
     */
    private $orderManagement;

    /**
     * @param GuestPaymentInformationManagementInterface $guestPaymentInformationManagement
     * @param OrderRepository $orderRepository
     * @param CartRepositoryInterface $quoteRepository
     * @param TransactionManagerInterface $transactionManager
     * @param TransactionInterface $transaction
     * @param CustomerInterface $customer
     * @param AdapterResponseFactory $adapterResponseFactory
     * @param StoreManagerInterface $storeManager
     * @param \Magento\Framework\Locale\ResolverInterface $resolver
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
     * @param \Magento\Sales\Api\OrderManagementInterface $orderManagement
     */
    public function __construct(
        GuestPaymentInformationManagementInterface $guestPaymentInformationManagement,
        OrderRepository $orderRepository,
        CartRepositoryInterface $quoteRepository,
        TransactionManagerInterface $transactionManager,
        TransactionInterface $transaction,
        CustomerInterface $customer,
        AdapterResponseFactory $adapterResponseFactory,
        StoreManagerInterface $storeManager,
        \Magento\Framework\Locale\ResolverInterface $resolver,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        \Magento\Sales\Api\OrderManagementInterface $orderManagement
    ) {
        $this->guestPaymentInformationManagement = $guestPaymentInformationManagement;
        $this->orderRepository = $orderRepository;
        $this->quoteRepository = $quoteRepository;
        $this->transactionManager = $transactionManager;
        $this->transaction = $transaction;
        $this->customer = $customer;
        $this->adapterResponseFactory = $adapterResponseFactory;
        $this->storeManager = $storeManager;
        $this->resolver = $resolver;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->orderManagement = $orderManagement;
    }

    /**
     * {@inheritdoc}
     */
    public function adaptPlaceOrder(
        $cartId,
        $email,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        $country,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        // Find orders already containing this quote (if any) and cancel those
        $orders = $this->orderCollectionFactory->create()
            ->addFieldToFilter('quote_id', $cartId)
            ->addFieldToFilter('status', ['neq' => 'canceled']);
        foreach ($orders as $order) {
            $this->orderManagement->cancel($order->getId());
        }

        // Create a new order
        $orderId = $this->guestPaymentInformationManagement->savePaymentInformationAndPlaceOrder(
            $cartId,
            $email,
            $paymentMethod,
            $billingAddress
        );
        $order = $this->orderRepository->get($orderId);
        $quote = $this->quoteRepository->get($order->getQuoteId());

        $transaction = $this->transaction;
        $transaction->setAmount(round($order->getGrandTotal(), 2))
            ->setCurrency($this->storeManager->getStore()->getCurrentCurrency()->getCode())
            ->setReference($order->getQuoteId());

        $customer = $this->customer;
        $customer->setIp($quote->getRemoteIp())
            ->setCountry($country)
            ->setLocale(substr($this->resolver->getLocale(), 0, 2));

        $transaction = $this->transactionManager->create($transaction, $customer);
        $additionalInformation = $quote->getPayment()->getAdditionalInformation();
        $additionalInformation['transaction_id'] = $transaction->getId();
        $additionalInformation['method_title'] = $quote->getPayment()->getMethod();

        $quote->getPayment()->setAdditionalInformation($additionalInformation);
        $quote->save();
        $order->getPayment()->setAdditionalInformation($additionalInformation);
        $order->save();

        $transaction->setOrderIds([$orderId]);
        $transaction->save();

        $response = $this->adapterResponseFactory->create();
        $response->setTransaction($transaction);
        $response->setIncrementId($order->getIncrementId());

        return $response;
    }
}
