<?php

namespace Makecommerce\Ecommerce\Plugin\Backend;

use Magento\Backend\Model\Auth\Session;
use Magento\Framework\App\Cache\Manager;
use Magento\Framework\App\Config\ValueFactory;
use Magento\Framework\Exception\LocalizedException;
use Makecommerce\Ecommerce\Helper\Updater as PaymentUpdater;
use Makecommerce\Ecommerce\Model\Shipping\Locations\Update as LocationUpdater;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

class SessionPlugin
{
    const XML_PATH_MAKECOMMERCE_ECOMMERCE_UPDATE_CRON_EXPR = 'crontab/default/jobs/makecommerce_update/schedule/cron_expr';

    /**
     * @var PaymentUpdater
     */
    private $paymentUpdater;

    /**
     * @var LocationUpdater
     */
    private $locationUpdater;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var ValueFactory
     */
    private $configValueFactory;

    /**
     * @var Manager
     */
    private $cacheManager;

    /**
     * @param PaymentUpdater $paymentUpdater
     * @param LocationUpdater $locationUpdater
     * @param ScopeConfigInterface $scopeConfig
     * @param ValueFactory $configValueFactory
     * @param Manager $cacheManager
     */
    public function __construct(
        PaymentUpdater $paymentUpdater,
        LocationUpdater $locationUpdater,
        ScopeConfigInterface $scopeConfig,
        ValueFactory $configValueFactory,
        Manager $cacheManager
    ){
        $this->paymentUpdater = $paymentUpdater;
        $this->locationUpdater = $locationUpdater;
        $this->scopeConfig = $scopeConfig;
        $this->configValueFactory = $configValueFactory;
        $this->cacheManager = $cacheManager;
    }

    /**
     * @param Session $subject
     * @param $result
     * @return mixed
     * @throws LocalizedException
     */
    public function afterProcessLogin(Session $subject, $result)
    {
        // Set up randomized update schedule to minimize request pileups
        $cron_expr = $this->scopeConfig
            ->getValue(
                self::XML_PATH_MAKECOMMERCE_ECOMMERCE_UPDATE_CRON_EXPR,
                ScopeInterface::SCOPE_WEBSITE
            );
        if (!$cron_expr) {
            // Set cron expression
            $this->configValueFactory->create()->load(
                self::XML_PATH_MAKECOMMERCE_ECOMMERCE_UPDATE_CRON_EXPR,
                'path'
            )->setValue(
                rand(0, 59) . ' ' . rand(0, 23) . ' * * *'
            )->setPath(
                self::XML_PATH_MAKECOMMERCE_ECOMMERCE_UPDATE_CRON_EXPR
            )->save();
            // Fetch payment methods
            $this->paymentUpdater->update(true);
            // Fetch parcel point location
            $this->locationUpdater->update();
            // Flush configuration cache
            $this->cacheManager->clean(['config']);
        }
        return $result;
    }
}
